# -*- coding: utf-8 -*-
"""The Logical Volume Manager (LVM) file entry implementation."""

from dfvfs.lib import definitions
from dfvfs.lib import errors
from dfvfs.lib import lvm_helper
from dfvfs.vfs import file_entry
from dfvfs.vfs import lvm_directory


class LVMFileEntry(file_entry.FileEntry):
  """File system file entry that uses pyvslvm."""

  TYPE_INDICATOR = definitions.TYPE_INDICATOR_LVM

  def __init__(
      self, resolver_context, file_system, path_spec, is_root=False,
      is_virtual=False, vslvm_logical_volume=None):
    """Initializes a file entry.

    Args:
      resolver_context (Context): resolver context.
      file_system (FileSystem): file system.
      path_spec (PathSpec): path specification.
      is_root (Optional[bool]): True if the file entry is the root file entry
          of the corresponding file system.
      is_virtual (Optional[bool]): True if the file entry is a virtual file
      vslvm_logical_volume (Optional[pyvslvm.logical_volume]): a LVM logical
          volume.

    Raises:
      BackEndError: when LVM logical volume is missing for a non-virtual file
          entry.
    """
    if not is_virtual and vslvm_logical_volume is None:
      vslvm_logical_volume = file_system.GetLVMLogicalVolumeByPathSpec(
          path_spec)
    if not is_virtual and vslvm_logical_volume is None:
      raise errors.BackEndError(
          'Missing vslvm logical volume in non-virtual file entry.')

    super(LVMFileEntry, self).__init__(
        resolver_context, file_system, path_spec, is_root=is_root,
        is_virtual=is_virtual)
    self._name = None
    self._vslvm_logical_volume = vslvm_logical_volume

    if self._is_virtual:
      self.entry_type = definitions.FILE_ENTRY_TYPE_DIRECTORY
    else:
      self.entry_type = definitions.FILE_ENTRY_TYPE_FILE

  def _GetDirectory(self):
    """Retrieves the directory.

    Returns:
      LVMDirectory: a directory.
    """
    if self.entry_type != definitions.FILE_ENTRY_TYPE_DIRECTORY:
      return None

    return lvm_directory.LVMDirectory(self._file_system, self.path_spec)

  def _GetSubFileEntries(self):
    """Retrieves sub file entries.

    Yields:
      LVMFileEntry: a sub file entry.
    """
    if self._directory is None:
      self._directory = self._GetDirectory()

    if self._directory:
      for path_spec in self._directory.entries:
        yield LVMFileEntry(self._resolver_context, self._file_system, path_spec)

  # TODO: implement creation_time property after implementing
  # vslvm_logical_volume.get_creation_time_as_integer()

  @property
  def name(self):
    """str: name of the file entry, without the full path."""
    if self._name is None:
      location = getattr(self.path_spec, 'location', None)
      if location is not None:
        self._name = self._file_system.BasenamePath(location)
      else:
        volume_index = getattr(self.path_spec, 'volume_index', None)
        if volume_index is not None:
          lvm_volume_index = volume_index + 1
          self._name = f'lvm{lvm_volume_index:d}'
        else:
          self._name = ''

    return self._name

  @property
  def size(self):
    """int: size of the file entry in bytes or None if not available."""
    if self._vslvm_logical_volume is None:
      return None

    return self._vslvm_logical_volume.size

  def GetLVMLogicalVolume(self):
    """Retrieves the LVM logical volume.

    Returns:
      pyvslvm.logical_volume: a LVM logical volume.
    """
    return self._vslvm_logical_volume

  def GetParentFileEntry(self):
    """Retrieves the parent file entry.

    Returns:
      LVMFileEntry: parent file entry or None if not available.
    """
    volume_index = lvm_helper.LVMPathSpecGetVolumeIndex(self.path_spec)
    if volume_index is None:
      return None

    return self._file_system.GetRootFileEntry()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the VFS file entry interface."""

import unittest

from dfvfs.lib import errors
from dfvfs.path import fake_path_spec
from dfvfs.resolver import context
from dfvfs.vfs import fake_file_system
from dfvfs.vfs import directory
from dfvfs.vfs import file_entry

from tests import test_lib as shared_test_lib


class TestFileEntry(file_entry.FileEntry):
  """File entry for testing."""

  # pylint: disable=abstract-method

  TYPE_INDICATOR = 'test'

  def _GetDirectory(self):
    """Retrieves the directory.

    Returns:
      Directory: a directory or None.
    """
    return directory.Directory(self._file_system, self.path_spec)

  def _GetSubFileEntries(self):
    """Retrieves sub file entries.

    Returns:
      generator[FileEntry]: a sub file entry generator.
    """
    return iter([])


class FileEntryTest(shared_test_lib.BaseTestCase):
  """Tests the VFS file entry interface."""

  # pylint: disable=protected-access

  def setUp(self):
    """Sets up the needed objects used throughout the test."""
    self._resolver_context = context.Context()
    self._path_spec = fake_path_spec.FakePathSpec(location='/')

    self._file_system = fake_file_system.FakeFileSystem(
        self._resolver_context, self._path_spec)
    self._file_system.Open()

  def tearDown(self):
    """Cleans up the needed objects used throughout the test."""
    self._resolver_context.Empty()

  def testIntialize(self):
    """Tests the __init__ function."""
    with self.assertRaises(ValueError):
      file_entry.FileEntry(
          self._resolver_context, self._file_system, self._path_spec)

  def testGetAttributes(self):
    """Tests the _GetAttributes function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    attributes = test_file_entry._GetAttributes()
    self.assertEqual(attributes, [])

  def testGetDataStreams(self):
    """Tests the _GetDataStreams function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    data_streams = test_file_entry._GetDataStreams()
    self.assertEqual(data_streams, [])

  def testGetLink(self):
    """Tests the _GetLink function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    symbolic_link = test_file_entry._GetLink()
    self.assertIsNotNone(symbolic_link)

  def testAccessTime(self):
    """Tests the access_time property."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertIsNone(test_file_entry.access_time)

  def testAddedTime(self):
    """Tests the added_time property."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertIsNone(test_file_entry.added_time)

  def testAttributes(self):
    """Tests the attributes property."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertEqual(test_file_entry.attributes, [])

  def testBackupTime(self):
    """Tests the backup_time property."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertIsNone(test_file_entry.backup_time)

  def testChangeTime(self):
    """Tests the change_time property."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertIsNone(test_file_entry.change_time)

  def testCreationTime(self):
    """Tests the creation_time property."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertIsNone(test_file_entry.creation_time)

  def testDeletionTime(self):
    """Tests the deletion_time property."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertIsNone(test_file_entry.deletion_time)

  def testDataStreams(self):
    """Tests the data_streams property."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertEqual(test_file_entry.data_streams, [])

  def testLink(self):
    """Tests the link property."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertIsNone(test_file_entry.link)

  def testModificationTime(self):
    """Tests the modification_time property."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertIsNone(test_file_entry.modification_time)

  def testName(self):
    """Tests the name property."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertIsNone(test_file_entry.name)

  def testNumberOfAttributes(self):
    """Tests the number_of_attributes property."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertEqual(test_file_entry.number_of_attributes, 0)

  def testNumberOfDataStreams(self):
    """Tests the number_of_data_streams property."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertEqual(test_file_entry.number_of_data_streams, 0)

  def testNumberOfSubFileEntries(self):
    """Tests the number_of_sub_file_entries property."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertEqual(test_file_entry.number_of_sub_file_entries, 0)

  def testSubFileEntries(self):
    """Tests the sub_file_entries property."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertIsNotNone(test_file_entry.sub_file_entries)

  def testTypeIndicator(self):
    """Tests the type_indicator property."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertEqual(test_file_entry.type_indicator, 'test')

  def testGetDataStream(self):
    """Tests the GetDataStream function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    data_stream = test_file_entry.GetDataStream('')
    self.assertIsNone(data_stream)

    with self.assertRaises(ValueError):
      test_file_entry.GetDataStream(0)

  def testGetExtents(self):
    """Tests the GetExtents function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    extents = test_file_entry.GetExtents()
    self.assertEqual(len(extents), 0)

  def testGetFileObject(self):
    """Tests the GetFileObject function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    file_object = test_file_entry.GetFileObject(data_stream_name='bogus')
    self.assertIsNone(file_object)

    with self.assertRaises(errors.NotSupported):
      test_file_entry.GetFileObject()

  def testGetFileSystem(self):
    """Tests the GetFileSystem function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    file_system = test_file_entry.GetFileSystem()
    self.assertIsNotNone(file_system)

  def testGetLinkedFileEntry(self):
    """Tests the GetLinkedFileEntry function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    linked_file_entry = test_file_entry.GetLinkedFileEntry()
    self.assertIsNone(linked_file_entry)

  def testGetParentFileEntry(self):
    """Tests the GetParentFileEntry function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    parent_file_entry = test_file_entry.GetParentFileEntry()
    self.assertIsNone(parent_file_entry)

  def testGetSubFileEntryByName(self):
    """Tests the GetSubFileEntryByName function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    sub_file_entry = test_file_entry.GetSubFileEntryByName('bogus')
    self.assertIsNone(sub_file_entry)

  def testHasDataStream(self):
    """Tests the HasDataStream function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertFalse(test_file_entry.HasDataStream(''))

    with self.assertRaises(ValueError):
      test_file_entry.HasDataStream(0)

  def testHasExternalData(self):
    """Tests the HasExternalData function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertFalse(test_file_entry.HasExternalData())

  def testIsAllocated(self):
    """Tests the IsAllocated function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertTrue(test_file_entry.IsAllocated())

  def testIsDevice(self):
    """Tests the IsDevice function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertFalse(test_file_entry.IsDevice())

  def testIsDirectory(self):
    """Tests the IsDirectory function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertFalse(test_file_entry.IsDirectory())

  def testIsFile(self):
    """Tests the IsFile function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertFalse(test_file_entry.IsFile())

  def testIsLink(self):
    """Tests the IsLink function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertFalse(test_file_entry.IsLink())

  def testIsLocked(self):
    """Tests the IsLocked function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertFalse(test_file_entry.IsLocked())

  def testIsPipe(self):
    """Tests the IsPipe function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertFalse(test_file_entry.IsPipe())

  def testIsRoot(self):
    """Tests the IsRoot function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertFalse(test_file_entry.IsRoot())

  def testIsSocket(self):
    """Tests the IsSocket function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertFalse(test_file_entry.IsSocket())

  def testIsVirtual(self):
    """Tests the IsVirtual function."""
    test_file_entry = TestFileEntry(
        self._resolver_context, self._file_system, self._path_spec)

    self.assertFalse(test_file_entry.IsVirtual())


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the encrypted stream file-like object."""

import os
import unittest

from dfvfs.file_io import encrypted_stream_io
from dfvfs.lib import definitions
from dfvfs.lib import errors
from dfvfs.path import factory as path_spec_factory
from dfvfs.resolver import context
from dfvfs.resolver import resolver

from tests.file_io import test_lib


class AESEncryptedStreamWithKeyChainTest(test_lib.PaddedSyslogTestCase):
  """Tests the AES encrypted stream file-like object.

  The credentials are passed via the key chain.
  """
  _AES_KEY = b'This is a key123'
  _AES_MODE = definitions.ENCRYPTION_MODE_CBC
  _AES_IV = b'This is an IV456'

  def setUp(self):
    """Sets up the needed objects used throughout the test."""
    self._resolver_context = context.Context()
    test_path = self._GetTestFilePath(['syslog.aes'])
    self._SkipIfPathNotExists(test_path)

    test_os_path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_OS, location=test_path)
    self._encrypted_stream_path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_ENCRYPTED_STREAM,
        encryption_method=definitions.ENCRYPTION_METHOD_AES,
        parent=test_os_path_spec)
    resolver.Resolver.key_chain.SetCredential(
        self._encrypted_stream_path_spec, 'key', self._AES_KEY)
    resolver.Resolver.key_chain.SetCredential(
        self._encrypted_stream_path_spec, 'initialization_vector',
        self._AES_IV)
    resolver.Resolver.key_chain.SetCredential(
        self._encrypted_stream_path_spec, 'cipher_mode', self._AES_MODE)
    self.padding_size = 1

  def tearDown(self):
    """Cleans up the needed objects used throughout the test."""
    self._resolver_context.Empty()

  def testOpenCloseFileObject(self):
    """Test the open and close functionality using a file-like object."""
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    self._TestGetSizeFileObject(file_object)

  def testOpenClosePathSpec(self):
    """Test the open and close functionality using a path specification."""
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    self._TestGetSizeFileObject(file_object)

  def testSeek(self):
    """Test the seek functionality."""
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    self._TestSeekFileObject(file_object)

    # TODO: Test SEEK_CUR after open.

    # Test SEEK_END after open.
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    file_object.seek(-10 - self.padding_size, os.SEEK_END)
    self.assertEqual(file_object.read(5), b'times')

  def testRead(self):
    """Test the read functionality."""
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    self._TestReadFileObject(file_object)


class AESEncryptedStreamTest(test_lib.PaddedSyslogTestCase):
  """The unit test for a AES encrypted stream file-like object.

  The credentials are passed via the path specification.
  """

  _AES_CIPHER_MODE = definitions.ENCRYPTION_MODE_CBC
  _AES_INITIALIZATION_VECTOR = b'This is an IV456'
  _AES_KEY = b'This is a key123'

  def setUp(self):
    """Sets up the needed objects used throughout the test."""
    self._resolver_context = context.Context()
    test_path = self._GetTestFilePath(['syslog.aes'])
    self._SkipIfPathNotExists(test_path)

    test_os_path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_OS, location=test_path)
    self._encrypted_stream_path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_ENCRYPTED_STREAM,
        cipher_mode=self._AES_CIPHER_MODE,
        encryption_method=definitions.ENCRYPTION_METHOD_AES,
        initialization_vector=self._AES_INITIALIZATION_VECTOR,
        key=self._AES_KEY, parent=test_os_path_spec)
    self.padding_size = 1

  def tearDown(self):
    """Cleans up the needed objects used throughout the test."""
    self._resolver_context.Empty()

  def testOpenCloseFileObject(self):
    """Test the open and close functionality using a file-like object."""
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    self._TestGetSizeFileObject(file_object)

  def testOpenClosePathSpec(self):
    """Test the open and close functionality using a path specification."""
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    self._TestGetSizeFileObject(file_object)

  def testSeek(self):
    """Test the seek functionality."""
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    self._TestSeekFileObject(file_object)

    # TODO: Test SEEK_CUR after open.

    # Test SEEK_END after open.
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    file_object.seek(-10 - self.padding_size, os.SEEK_END)
    self.assertEqual(file_object.read(5), b'times')

  def testRead(self):
    """Test the read functionality."""
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    self._TestReadFileObject(file_object)


class BlowfishEncryptedStreamWithKeyChainTest(test_lib.PaddedSyslogTestCase):
  """Tests the Blowfish encrypted stream file-like object.

  The credentials are passed via the key chain.
  """
  _BLOWFISH_KEY = b'This is a key123'
  _BLOWFISH_MODE = definitions.ENCRYPTION_MODE_CBC
  _BLOWFISH_IV = b'This IV!'

  def setUp(self):
    """Sets up the needed objects used throughout the test."""
    self._resolver_context = context.Context()
    test_path = self._GetTestFilePath(['syslog.blowfish'])
    self._SkipIfPathNotExists(test_path)

    test_os_path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_OS, location=test_path)
    self._encrypted_stream_path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_ENCRYPTED_STREAM,
        encryption_method=definitions.ENCRYPTION_METHOD_BLOWFISH,
        parent=test_os_path_spec)
    resolver.Resolver.key_chain.SetCredential(
        self._encrypted_stream_path_spec, 'key', self._BLOWFISH_KEY)
    resolver.Resolver.key_chain.SetCredential(
        self._encrypted_stream_path_spec, 'initialization_vector',
        self._BLOWFISH_IV)
    resolver.Resolver.key_chain.SetCredential(
        self._encrypted_stream_path_spec, 'cipher_mode', self._BLOWFISH_MODE)
    self.padding_size = 1

  def tearDown(self):
    """Cleans up the needed objects used throughout the test."""
    self._resolver_context.Empty()

  def testOpenCloseFileObject(self):
    """Test the open and close functionality using a file-like object."""
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    self._TestGetSizeFileObject(file_object)

  def testOpenClosePathSpec(self):
    """Test the open and close functionality using a path specification."""
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    self._TestGetSizeFileObject(file_object)

  def testSeek(self):
    """Test the seek functionality."""
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    self._TestSeekFileObject(file_object)

    # TODO: Test SEEK_CUR after open.

    # Test SEEK_END after open.
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    file_object.seek(-10 - self.padding_size, os.SEEK_END)
    self.assertEqual(file_object.read(5), b'times')

  def testRead(self):
    """Test the read functionality."""
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    self._TestReadFileObject(file_object)


class DES3EncryptedStreamWithKeyChainTest(test_lib.PaddedSyslogTestCase):
  """Tests the Triple DES encrypted stream file-like object.

  The credentials are passed via the key chain.
  """
  _DES3_KEY = b'This is a key123'
  _DES3_MODE = definitions.ENCRYPTION_MODE_CBC
  _DES3_IV = b'This IV!'

  def setUp(self):
    """Sets up the needed objects used throughout the test."""
    self._resolver_context = context.Context()
    test_path = self._GetTestFilePath(['syslog.des3'])
    self._SkipIfPathNotExists(test_path)

    test_os_path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_OS, location=test_path)
    self._encrypted_stream_path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_ENCRYPTED_STREAM,
        encryption_method=definitions.ENCRYPTION_METHOD_DES3,
        parent=test_os_path_spec)
    resolver.Resolver.key_chain.SetCredential(
        self._encrypted_stream_path_spec, 'key', self._DES3_KEY)
    resolver.Resolver.key_chain.SetCredential(
        self._encrypted_stream_path_spec, 'initialization_vector',
        self._DES3_IV)
    resolver.Resolver.key_chain.SetCredential(
        self._encrypted_stream_path_spec, 'cipher_mode', self._DES3_MODE)
    self.padding_size = 1

  def tearDown(self):
    """Cleans up the needed objects used throughout the test."""
    self._resolver_context.Empty()

  def testOpenCloseFileObject(self):
    """Test the open and close functionality using a file-like object."""
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    self._TestGetSizeFileObject(file_object)

  def testOpenClosePathSpec(self):
    """Test the open and close functionality using a path specification."""
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    self._TestGetSizeFileObject(file_object)

  def testSeek(self):
    """Test the seek functionality."""
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    self._TestSeekFileObject(file_object)

    # TODO: Test SEEK_CUR after open.

    # Test SEEK_END after open.
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    file_object.seek(-10 - self.padding_size, os.SEEK_END)
    self.assertEqual(file_object.read(5), b'times')

  def testRead(self):
    """Test the read functionality."""
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    self._TestReadFileObject(file_object)


class RC4EncryptedStreamWithKeyChainTest(test_lib.SylogTestCase):
  """Tests the RC4 encrypted stream file-like object.

  The credentials are passed via the key chain.
  """
  _RC4_KEY = b'rc4test'

  def setUp(self):
    """Sets up the needed objects used throughout the test."""
    self._resolver_context = context.Context()
    test_path = self._GetTestFilePath(['syslog.rc4'])
    self._SkipIfPathNotExists(test_path)

    test_os_path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_OS, location=test_path)
    self._encrypted_stream_path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_ENCRYPTED_STREAM,
        encryption_method=definitions.ENCRYPTION_METHOD_RC4,
        parent=test_os_path_spec)
    resolver.Resolver.key_chain.SetCredential(
        self._encrypted_stream_path_spec, 'key', self._RC4_KEY)

  def tearDown(self):
    """Cleans up the needed objects used throughout the test."""
    self._resolver_context.Empty()

  def testOpenCloseFileObject(self):
    """Test the open and close functionality using a file-like object."""
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    try:
      self._TestGetSizeFileObject(file_object)
    except errors.BackEndError:
      raise unittest.SkipTest('missing cryptograpy support')

  def testOpenClosePathSpec(self):
    """Test the open and close functionality using a path specification."""
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    try:
      self._TestGetSizeFileObject(file_object)
    except errors.BackEndError:
      raise unittest.SkipTest('missing cryptograpy support')

  def testSeek(self):
    """Test the seek functionality."""
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    try:
      self._TestSeekFileObject(file_object)
    except errors.BackEndError:
      raise unittest.SkipTest('missing cryptograpy support')

    # TODO: Test SEEK_CUR after open.

    # Test SEEK_END after open.
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    file_object.seek(-10, os.SEEK_END)
    self.assertEqual(file_object.read(5), b'times')

  def testRead(self):
    """Test the read functionality."""
    file_object = encrypted_stream_io.EncryptedStream(
        self._resolver_context, self._encrypted_stream_path_spec)
    file_object.Open()

    try:
      self._TestReadFileObject(file_object)
    except errors.BackEndError:
      raise unittest.SkipTest('missing cryptograpy support')


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The operating system file entry implementation."""

import os
import platform
import stat

import pysmdev

try:
  import xattr
except ImportError:
  xattr = None

from dfdatetime import posix_time as dfdatetime_posix_time

from dfvfs.lib import definitions
from dfvfs.path import os_path_spec
from dfvfs.vfs import attribute
from dfvfs.vfs import file_entry
from dfvfs.vfs import os_attribute
from dfvfs.vfs import os_directory


class OSFileEntry(file_entry.FileEntry):
  """File system file entry that uses os."""

  TYPE_INDICATOR = definitions.TYPE_INDICATOR_OS

  _OS_IS_WINDOWS = platform.system() == 'Windows'

  def __init__(self, resolver_context, file_system, path_spec, is_root=False):
    """Initializes a file entry.

    Args:
      resolver_context (Context): resolver context.
      file_system (FileSystem): file system.
      path_spec (PathSpec): path specification.
      is_root (Optional[bool]): True if the file entry is the root file entry
          of the corresponding file system.
    """
    location = getattr(path_spec, 'location', None)

    # Windows does not support running os.stat on device files so we use
    # libsmdev to do an initial check.
    is_windows_device = False
    if self._OS_IS_WINDOWS and location:
      try:
        # pylint: disable=no-member
        is_windows_device = pysmdev.check_device(location)
      except IOError:
        pass

    stat_info = None
    if not is_windows_device and location:
      # We are only catching OSError. However on the Windows platform
      # a WindowsError can be raised as well. We are not catching that since
      # that error does not exist on non-Windows platforms.
      try:
        stat_info = os.lstat(location)
      except (IOError, OSError):
        stat_info = None

    super(OSFileEntry, self).__init__(
        resolver_context, file_system, path_spec, is_root=is_root,
        is_virtual=False)
    self._is_windows_device = is_windows_device
    self._location = location
    self._name = None
    self._stat_info = stat_info

    if is_windows_device:
      self.entry_type = definitions.FILE_ENTRY_TYPE_DEVICE

    elif stat_info:
      # If location contains a trailing segment separator and points to
      # a symbolic link to a directory stat info will not indicate
      # the file entry as a symbolic link. The following check ensures
      # that the LINK type is correctly detected.
      is_link = os.path.islink(location)

      # The stat info member st_mode can have multiple types e.g.
      # LINK and DIRECTORY in case of a symbolic link to a directory
      # dfVFS currently only supports one type so we need to check
      # for LINK first.
      if stat.S_ISLNK(stat_info.st_mode) or is_link:
        self.entry_type = definitions.FILE_ENTRY_TYPE_LINK
      elif stat.S_ISREG(stat_info.st_mode):
        self.entry_type = definitions.FILE_ENTRY_TYPE_FILE
      elif stat.S_ISDIR(stat_info.st_mode):
        self.entry_type = definitions.FILE_ENTRY_TYPE_DIRECTORY
      elif stat.S_ISCHR(stat_info.st_mode):
        self.entry_type = definitions.FILE_ENTRY_TYPE_CHARACTER_DEVICE
      elif stat.S_ISBLK(stat_info.st_mode):
        self.entry_type = definitions.FILE_ENTRY_TYPE_BLOCK_DEVICE
      elif stat.S_ISFIFO(stat_info.st_mode):
        self.entry_type = definitions.FILE_ENTRY_TYPE_PIPE
      elif stat.S_ISSOCK(stat_info.st_mode):
        self.entry_type = definitions.FILE_ENTRY_TYPE_SOCKET

  def _GetAttributes(self):
    """Retrieves the attributes.

    Returns:
      list[Attribute]: attributes.
    """
    if self._attributes is None:
      self._attributes = []

      if xattr:
        for name in xattr.listxattr(self._location):
          if isinstance(name, bytes):
            name = os.fsdecode(name)

          extended_attribute = os_attribute.OSExtendedAttribute(
              self._location, name)
          self._attributes.append(extended_attribute)

    return self._attributes

  def _GetDirectory(self):
    """Retrieves a directory.

    Returns:
      OSDirectory: a directory.
    """
    if self.entry_type != definitions.FILE_ENTRY_TYPE_DIRECTORY:
      return None

    return os_directory.OSDirectory(self._file_system, self.path_spec)

  def _GetLink(self):
    """Retrieves the link.

    Returns:
      str: full path of the linked file entry.
    """
    if self._link is None:
      self._link = ''

      if self._location is None:
        return self._link

      self._link = os.readlink(self._location)
      self._link = os.path.abspath(self._link)

    return self._link

  def _GetStatAttribute(self):
    """Retrieves a stat attribute.

    Returns:
      StatAttribute: a stat attribute or None if not available.
    """
    stat_attribute = attribute.StatAttribute()

    if not self._is_windows_device and self._stat_info:
      stat_attribute.group_identifier = self._stat_info.st_gid
      stat_attribute.inode_number = self._stat_info.st_ino
      stat_attribute.mode = stat.S_IMODE(self._stat_info.st_mode)
      stat_attribute.number_of_links = self._stat_info.st_nlink
      stat_attribute.owner_identifier = self._stat_info.st_uid
      stat_attribute.size = self._stat_info.st_size

    stat_attribute.type = self.entry_type

    return stat_attribute

  def _GetSubFileEntries(self):
    """Retrieves sub file entries.

    Yields:
      OSFileEntry: a sub file entry.
    """
    if self._directory is None:
      self._directory = self._GetDirectory()

    if self._directory:
      for path_spec in self._directory.entries:
        yield OSFileEntry(self._resolver_context, self._file_system, path_spec)

  @property
  def access_time(self):
    """dfdatetime.DateTimeValues: access time or None if not available."""
    if self._stat_info is None:
      return None

    timestamp = getattr(self._stat_info, 'st_atime_ns', None)
    if timestamp is not None:
      return dfdatetime_posix_time.PosixTimeInNanoseconds(timestamp=timestamp)

    timestamp = int(self._stat_info.st_atime)
    return dfdatetime_posix_time.PosixTime(timestamp=timestamp)

  @property
  def change_time(self):
    """dfdatetime.DateTimeValues: change time or None if not available."""
    if self._stat_info is None:
      return None

    # Per Python os.stat() documentation the value of stat_results.st_ctime
    # contains the creation time on Windows.
    if self._OS_IS_WINDOWS:
      return None

    timestamp = getattr(self._stat_info, 'st_ctime_ns', None)
    if timestamp is not None:
      return dfdatetime_posix_time.PosixTimeInNanoseconds(timestamp=timestamp)

    timestamp = int(self._stat_info.st_ctime)
    return dfdatetime_posix_time.PosixTime(timestamp=timestamp)

  @property
  def creation_time(self):
    """dfdatetime.DateTimeValues: creation time or None if not available."""
    if self._stat_info is None:
      return None

    # Per Python os.stat() documentation the value of stat_results.st_ctime
    # contains the creation time on Windows.
    if self._OS_IS_WINDOWS:
      timestamp = getattr(self._stat_info, 'st_ctime_ns', None)
      if timestamp is not None:
        return dfdatetime_posix_time.PosixTimeInNanoseconds(timestamp=timestamp)

      timestamp = int(self._stat_info.st_ctime)

    else:
      timestamp = getattr(self._stat_info, 'st_birthtime', None)
      if timestamp is None:
        return None

    return dfdatetime_posix_time.PosixTime(timestamp=timestamp)

  @property
  def modification_time(self):
    """dfdatetime.DateTimeValues: modification time or None if not available."""
    if self._stat_info is None:
      return None

    timestamp = getattr(self._stat_info, 'st_mtime_ns', None)
    if timestamp is not None:
      return dfdatetime_posix_time.PosixTimeInNanoseconds(timestamp=timestamp)

    timestamp = int(self._stat_info.st_mtime)
    return dfdatetime_posix_time.PosixTime(timestamp=timestamp)

  @property
  def name(self):
    """str: name of the file entry, without the full path."""
    if self._name is None:
      if self._location is not None:
        self._name = self._file_system.BasenamePath(self._location)
    return self._name

  @property
  def size(self):
    """int: size of the file entry in bytes or None if not available."""
    if self._is_windows_device or not self._stat_info:
      return None

    return self._stat_info.st_size

  def GetLinkedFileEntry(self):
    """Retrieves the linked file entry, for example for a symbolic link.

    Returns:
      OSFileEntry: linked file entry or None if not available.
    """
    link = self._GetLink()
    if not link:
      return None

    path_spec = os_path_spec.OSPathSpec(location=link)
    return OSFileEntry(self._resolver_context, self._file_system, path_spec)

  def GetParentFileEntry(self):
    """Retrieves the parent file entry.

    Returns:
      OSFileEntry: parent file entry or None if not available.
    """
    if self._location is None:
      return None

    parent_location = self._file_system.DirnamePath(self._location)
    if parent_location is None:
      return None

    if parent_location == '':
      parent_location = self._file_system.PATH_SEPARATOR

    path_spec = os_path_spec.OSPathSpec(location=parent_location)
    return OSFileEntry(self._resolver_context, self._file_system, path_spec)



# -*- coding: utf-8 -*-
"""The ZIP directory implementation."""

from dfvfs.path import zip_path_spec
from dfvfs.vfs import directory


class ZIPDirectory(directory.Directory):
  """File system directory that uses zipfile."""

  def _EntriesGenerator(self):
    """Retrieves directory entries.

    Since a directory can contain a vast number of entries using
    a generator is more memory efficient.

    Yields:
      ZipPathSpec: a path specification.
    """
    location = getattr(self.path_spec, 'location', None)

    if location and location.startswith(self._file_system.PATH_SEPARATOR):
      # The zip_info filename does not have the leading path separator
      # as the location string does.
      zip_path = location[1:]

      # Set of top level sub directories that have been yielded.
      processed_directories = set()

      zip_file = self._file_system.GetZipFile()
      for zip_info in zip_file.infolist():
        path = getattr(zip_info, 'filename', None)
        if path is not None and not isinstance(path, str):
          try:
            path = path.decode(self._file_system.encoding)
          except UnicodeDecodeError:
            path = None

        if not path or not path.startswith(zip_path):
          continue

        # Ignore the directory itself.
        if path == zip_path:
          continue

        path_segment, suffix = self._file_system.GetPathSegmentAndSuffix(
            zip_path, path)
        if not path_segment:
          continue

        # Some times the ZIP file lacks directories, therefore we will
        # provide virtual ones.
        if suffix:
          path_spec_location = self._file_system.JoinPath([
              location, path_segment])
          is_directory = True
        else:
          path_spec_location = self._file_system.JoinPath([path])
          is_directory = path.endswith('/')

        if is_directory:
          if path_spec_location in processed_directories:
            continue
          processed_directories.add(path_spec_location)
          # Restore / at end path to indicate a directory.
          path_spec_location += self._file_system.PATH_SEPARATOR

        yield zip_path_spec.ZipPathSpec(
            location=path_spec_location, parent=self.path_spec.parent)



# -*- coding: utf-8 -*-
"""The decrypter interface."""

import abc

from cryptography.hazmat import backends
from cryptography.hazmat.primitives import ciphers
from cryptography.hazmat.primitives.ciphers import modes

from dfvfs.lib import definitions
from dfvfs.lib import errors


class Decrypter(object):
  """Decrypter interface."""

  def __init__(self, **kwargs):
    """Initializes a decrypter.

    Args:
      kwargs (dict): keyword arguments depending on the decrypter.

    Raises:
      ValueError: when there are unused keyword arguments.
    """
    if kwargs:
      keyword_arguments = ', '.join(kwargs)
      raise ValueError(f'Unused keyword arguments: {keyword_arguments:s}.')

    super(Decrypter, self).__init__()

  # pylint: disable=redundant-returns-doc
  @abc.abstractmethod
  def Decrypt(self, encrypted_data, finalize=False):
    """Decrypts the encrypted data.

    Args:
      encrypted_data (bytes): encrypted data.
      finalize (Optional[bool]): True if the end of data has been reached and
          the cipher context should be finalized.

    Returns:
      tuple[bytes, bytes]: decrypted data and remaining encrypted data.
    """


class CryptographyBlockCipherDecrypter(Decrypter):
  """Block cipher decrypter using Cryptography."""

  def __init__(
      self, algorithm=None, cipher_mode=None, initialization_vector=None,
      **kwargs):
    """Initializes a decrypter.

    Args:
      algorithm (Optional[Cryptography.CipherAlgorithm]): cipher algorithm.
      cipher_mode (Optional[str]): cipher mode.
      initialization_vector (Optional[bytes]): initialization vector.
      kwargs (dict): keyword arguments depending on the decrypter.

    Raises:
      BackEndError: when the cryptography backend cannot be initialized.
      ValueError: if the initialization_vector is required and not set.
    """
    if (cipher_mode != definitions.ENCRYPTION_MODE_ECB and
        not initialization_vector):
      raise ValueError('Missing initialization vector.')

    if cipher_mode == definitions.ENCRYPTION_MODE_CBC:
      mode = modes.CBC(initialization_vector)
    elif cipher_mode == definitions.ENCRYPTION_MODE_CFB:
      mode = modes.CFB(initialization_vector)
    elif cipher_mode == definitions.ENCRYPTION_MODE_ECB:
      mode = modes.ECB()
    elif cipher_mode == definitions.ENCRYPTION_MODE_OFB:
      mode = modes.OFB(initialization_vector)

    try:
      backend = backends.default_backend()
    except ImportError as exception:
      raise errors.BackEndError(exception)

    cipher = ciphers.Cipher(algorithm, mode=mode, backend=backend)

    super(CryptographyBlockCipherDecrypter, self).__init__()
    self._algorithm = algorithm
    self._cipher_context = cipher.decryptor()

  def Decrypt(self, encrypted_data, finalize=False):
    """Decrypts the encrypted data.

    Args:
      encrypted_data (bytes): encrypted data.
      finalize (Optional[bool]): True if the end of data has been reached and
          the cipher context should be finalized.

    Returns:
      tuple[bytes, bytes]: decrypted data and remaining encrypted data.
    """
    encrypted_data_size = len(encrypted_data)

    _, block_offset = divmod(encrypted_data_size, self._algorithm.block_size)

    remaining_encrypted_data = b''
    if not finalize and block_offset > 0:
      remaining_encrypted_data = encrypted_data[-block_offset:]
      encrypted_data = encrypted_data[:-block_offset]

    decrypted_data = self._cipher_context.update(encrypted_data)

    if finalize:
      if block_offset > 0:
        block_padding_size = self._algorithm.block_size - block_offset
        block_padding = b'\x00' * block_padding_size

        decrypted_data += self._cipher_context.update(block_padding)

      decrypted_data += self._cipher_context.finalize()
      decrypted_data = decrypted_data[:encrypted_data_size]

    return decrypted_data, remaining_encrypted_data



# -*- coding: utf-8 -*-
"""The resolver context object."""

import weakref

from dfvfs.mount import manager as mount_manager


class Context(object):
  """Resolver context."""

  def __init__(self):
    """Initializes the resolver context."""
    super(Context, self).__init__()
    # The WeakValueDictionary will maintain a (weak) reference to a VFS object
    # as long as the object is (strong) referrened by other objects. If an
    # object has no remaining (strong) references it is removed from the
    # WeakValueDictionary.
    self._file_object_cache = weakref.WeakValueDictionary()
    self._file_system_cache = weakref.WeakValueDictionary()
    self._mount_points = {}

  def _GetFileSystemCacheIdentifier(self, path_spec):
    """Determines the file system cache identifier for the path specification.

    Args:
      path_spec (PathSpec): path specification.

    Returns:
      str: identifier of the VFS object.
    """
    string_parts = []

    string_parts.append(getattr(path_spec.parent, 'comparable', ''))
    string_parts.append(f'type: {path_spec.type_indicator:s}')

    return ''.join(string_parts)

  def DeregisterMountPoint(self, mount_point):
    """Deregisters a path specification mount point.

    Args:
      mount_point (str): mount point identifier.

    Raises:
      KeyError: if the corresponding mount point is not set.
    """
    if mount_point not in self._mount_points:
      raise KeyError(f'Mount point: {mount_point:s} not set.')

    del self._mount_points[mount_point]

  def CacheFileObject(self, path_spec, file_object):
    """Caches a file-like object based on a path specification.

    Args:
      path_spec (PathSpec): path specification.
      file_object (FileIO): file-like object.

    Raises:
      KeyError: if the file object already is cached.
    """
    identifier = path_spec.comparable

    if identifier in self._file_object_cache:
      raise KeyError(
          f'File object already cached for identifier: {identifier:s}')

    self._file_object_cache[identifier] = file_object

  def CacheFileSystem(self, path_spec, file_system):
    """Caches a file system object based on a path specification.

    Args:
      path_spec (PathSpec): path specification.
      file_system (FileSystem): file system object.

    Raises:
      KeyError: if the file system already is cached.
    """
    identifier = self._GetFileSystemCacheIdentifier(path_spec)

    if identifier in self._file_system_cache:
      raise KeyError(
          f'File system already cached for identifier: {identifier:s}')

    self._file_system_cache[identifier] = file_system

  def Empty(self):
    """Empties the caches."""
    self._file_object_cache.clear()
    self._file_system_cache.clear()

  def GetFileObject(self, path_spec):
    """Retrieves a file-like object defined by path specification.

    Args:
      path_spec (PathSpec): path specification.

    Returns:
      FileIO: a file-like object or None if not cached.
    """
    return self._file_object_cache.get(path_spec.comparable, None)

  def GetFileSystem(self, path_spec):
    """Retrieves a file system object defined by path specification.

    Args:
      path_spec (PathSpec): path specification.

    Returns:
      FileSystem: a file system object or None if not cached.
    """
    identifier = self._GetFileSystemCacheIdentifier(path_spec)
    return self._file_system_cache.get(identifier, None)

  def GetMountPoint(self, mount_point):
    """Retrieves the path specification of a mount point.

    Args:
      mount_point (str): mount point identifier.

    Returns:
      PathSpec: path specification of the mount point or None if the mount
          point does not exists.
    """
    path_spec = self._mount_points.get(mount_point, None)
    if not path_spec:
      path_spec = mount_manager.MountPointManager.GetMountPoint(mount_point)
    return path_spec

  def RegisterMountPoint(self, mount_point, path_spec):
    """Registers a path specification mount point.

    Args:
      mount_point (str): mount point identifier.
      path_spec (PathSpec): path specification of the mount point.

    Raises:
      KeyError: if the corresponding mount point is already set.
    """
    if mount_point in self._mount_points:
      raise KeyError(f'Mount point: {mount_point:s} already set.')

    self._mount_points[mount_point] = path_spec



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the file entry implementation using SQLite blob."""

import unittest

from dfvfs.lib import definitions
from dfvfs.path import factory as path_spec_factory
from dfvfs.resolver import context
from dfvfs.vfs import sqlite_blob_file_entry
from dfvfs.vfs import sqlite_blob_file_system

from tests import test_lib as shared_test_lib


class SQLiteBlobFileEntryTest(shared_test_lib.BaseTestCase):
  """Tests for the SQLite blob file entry."""

  def setUp(self):
    """Sets up the needed objects used throughout the test."""
    self._resolver_context = context.Context()
    test_path = self._GetTestFilePath(['blob.db'])
    self._SkipIfPathNotExists(test_path)

    test_os_path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_OS, location=test_path)
    self._sqlite_blob_path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_SQLITE_BLOB, column_name='blobs',
        parent=test_os_path_spec, row_condition=('name', '==', 'mmssms.db'),
        table_name='myblobs')
    self._sqlite_blob_path_spec_2 = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_SQLITE_BLOB, column_name='blobs',
        parent=test_os_path_spec, row_index=2, table_name='myblobs')
    self._sqlite_blob_path_spec_3 = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_SQLITE_BLOB, column_name='blobs',
        parent=test_os_path_spec, row_condition=('name', '==', 4),
        table_name='myblobs')
    self._sqlite_blob_path_spec_directory = (
        path_spec_factory.Factory.NewPathSpec(
            definitions.TYPE_INDICATOR_SQLITE_BLOB, column_name='blobs',
            parent=test_os_path_spec, table_name='myblobs'))

    self._file_system = sqlite_blob_file_system.SQLiteBlobFileSystem(
        self._resolver_context, self._sqlite_blob_path_spec)
    self._file_system.Open()

  def tearDown(self):
    """Cleans up the needed objects used throughout the test."""
    self._resolver_context.Empty()

  def testIntialize(self):
    """Test the __init__ function."""
    file_entry = sqlite_blob_file_entry.SQLiteBlobFileEntry(
        self._resolver_context, self._file_system, self._sqlite_blob_path_spec)

    self.assertIsNotNone(file_entry)

    file_entry = sqlite_blob_file_entry.SQLiteBlobFileEntry(
        self._resolver_context, self._file_system,
        self._sqlite_blob_path_spec_2)

    self.assertIsNotNone(file_entry)

  # TODO: add tests for _GetDirectory
  # TODO: add tests for _GetSubFileEntries

  def testName(self):
    """Test name property."""
    file_entry = self._file_system.GetFileEntryByPathSpec(
        self._sqlite_blob_path_spec)
    self.assertTrue(file_entry.name == (
        'WHERE name == \'mmssms.db\''))

    file_entry = self._file_system.GetFileEntryByPathSpec(
        self._sqlite_blob_path_spec_3)
    self.assertTrue(file_entry.name == 'WHERE name == 4')

    file_entry = self._file_system.GetFileEntryByPathSpec(
        self._sqlite_blob_path_spec_directory)
    self.assertTrue(file_entry.name == 'myblobs.blobs')

  def testSize(self):
    """Test the size property."""
    file_entry = sqlite_blob_file_entry.SQLiteBlobFileEntry(
        self._resolver_context, self._file_system, self._sqlite_blob_path_spec)

    self.assertIsNotNone(file_entry)
    self.assertEqual(file_entry.size, 110592)

  # TODO: add tests for GetNumberOfRows

  def testGetFileEntryByPathSpec(self):
    """Test the get a file entry by path specification functionality."""
    file_entry = self._file_system.GetFileEntryByPathSpec(
        self._sqlite_blob_path_spec)

    self.assertIsNotNone(file_entry)

    file_entry = self._file_system.GetFileEntryByPathSpec(
        self._sqlite_blob_path_spec_2)

    self.assertIsNotNone(file_entry)

  def testGetParentFileEntry(self):
    """Tests the GetParentFileEntry function."""
    file_entry = self._file_system.GetFileEntryByPathSpec(
        self._sqlite_blob_path_spec)

    self.assertIsNotNone(file_entry)

    parent_file_entry = file_entry.GetParentFileEntry()

    self.assertIsNotNone(parent_file_entry)

    self.assertEqual(parent_file_entry.name, 'myblobs.blobs')

  def testIsFunctions(self):
    """Test the Is? functions."""
    file_entry = self._file_system.GetFileEntryByPathSpec(
        self._sqlite_blob_path_spec)

    self.assertFalse(file_entry.IsRoot())
    self.assertFalse(file_entry.IsVirtual())
    self.assertTrue(file_entry.IsAllocated())

    self.assertFalse(file_entry.IsDevice())
    self.assertFalse(file_entry.IsDirectory())
    self.assertTrue(file_entry.IsFile())
    self.assertFalse(file_entry.IsLink())
    self.assertFalse(file_entry.IsPipe())
    self.assertFalse(file_entry.IsSocket())

  def testSubFileEntries(self):
    """Test the sub file entries iteration functionality."""
    file_entry = self._file_system.GetFileEntryByPathSpec(
        self._sqlite_blob_path_spec_directory)

    self.assertTrue(file_entry.IsDirectory())

    self.assertIsNotNone(file_entry)

    self.assertEqual(file_entry.number_of_sub_file_entries, 4)

    expected_sub_file_entry_names = [
        'OFFSET 0', 'OFFSET 1', 'OFFSET 2', 'OFFSET 3']

    sub_file_entry_names = []
    for sub_file_entry in file_entry.sub_file_entries:
      sub_file_entry_names.append(sub_file_entry.name)

    self.assertEqual(
        len(sub_file_entry_names), len(expected_sub_file_entry_names))
    self.assertEqual(
        sorted(sub_file_entry_names), expected_sub_file_entry_names)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The SleuthKit (TSK) partition path specification implementation."""

from dfvfs.lib import definitions
from dfvfs.path import factory
from dfvfs.path import path_spec


class TSKPartitionPathSpec(path_spec.PathSpec):
  """SleuthKit (TSK) partition path specification.

  Attributes:
    location (str): location.
    part_index (int): part index.
    start_offset (int): start offset.
  """

  TYPE_INDICATOR = definitions.TYPE_INDICATOR_TSK_PARTITION

  def __init__(
      self, location=None, parent=None, part_index=None, start_offset=None,
      **kwargs):
    """Initializes a path specification.

    Note that the TSK partition path specification must have a parent.

    Args:
      location (Optional[str]): location.
      parent (Optional[PathSpec]): parent path specification.
      part_index (Optional[int]): part index.
      start_offset (Optional[int]): start offset.

    Raises:
      ValueError: when parent is not set.
    """
    if not parent:
      raise ValueError('Missing parent value.')

    super(TSKPartitionPathSpec, self).__init__(parent=parent, **kwargs)
    self.location = location
    self.part_index = part_index
    self.start_offset = start_offset

  @property
  def comparable(self):
    """str: comparable representation of the path specification."""
    string_parts = []

    if self.location is not None:
      string_parts.append(f'location: {self.location:s}')
    if self.part_index is not None:
      string_parts.append(f'part index: {self.part_index:d}')
    if self.start_offset is not None:
      string_parts.append(f'start offset: 0x{self.start_offset:08x}')

    return self._GetComparable(sub_comparable_string=', '.join(string_parts))


factory.Factory.RegisterPathSpec(TSKPartitionPathSpec)



# -*- coding: utf-8 -*-
"""The EXT attribute implementation."""

import os

from dfvfs.lib import definitions
from dfvfs.lib import errors
from dfvfs.vfs import attribute
from dfvfs.vfs import extent


class EXTExtendedAttribute(attribute.Attribute):
  """EXT extended attribute that uses pyfsext."""

  def __init__(self, fsext_extended_attribute):
    """Initializes an attribute.

    Args:
      fsext_extended_attribute (pyfsext.extended_attribute): EXT extended
          attribute.

    Raises:
      BackEndError: if the pyfsext extended attribute is missing.
    """
    if not fsext_extended_attribute:
      raise errors.BackEndError('Missing pyfsext extended attribute.')

    super(EXTExtendedAttribute, self).__init__()
    self._fsext_extended_attribute = fsext_extended_attribute

  @property
  def name(self):
    """str: name."""
    return self._fsext_extended_attribute.name

  def GetExtents(self):
    """Retrieves the extents.

    Returns:
      list[Extent]: the extents of the attribute data.
    """
    extents = []
    for extent_index in range(
        self._fsext_extended_attribute.number_of_extents):
      extent_offset, extent_size, extent_flags = (
          self._fsext_extended_attribute.get_extent(extent_index))

      if extent_flags & 0x1:
        extent_type = definitions.EXTENT_TYPE_SPARSE
      else:
        extent_type = definitions.EXTENT_TYPE_DATA

      data_stream_extent = extent.Extent(
          extent_type=extent_type, offset=extent_offset, size=extent_size)
      extents.append(data_stream_extent)

    return extents

  # Note: that the following functions do not follow the style guide
  # because they are part of the file-like object interface.
  # pylint: disable=invalid-name

  def read(self, size=None):
    """Reads a byte string from the file input/output (IO) object.

    The function will read a byte string of the specified size or
    all of the remaining data if no size was specified.

    Args:
      size (Optional[int]): number of bytes to read, where None is all
          remaining data.

    Returns:
      bytes: data read.

    Raises:
      IOError: if the read failed.
      OSError: if the read failed.
    """
    return self._fsext_extended_attribute.read_buffer(size)

  def seek(self, offset, whence=os.SEEK_SET):
    """Seeks to an offset within the file input/output (IO) object.

    Args:
      offset (int): offset to seek.
      whence (Optional[int]): value that indicates whether offset is an
          absolute or relative position within the file.

    Raises:
      IOError: if the seek failed.
      OSError: if the seek failed.
    """
    self._fsext_extended_attribute.seek_offset(offset, whence)

  # get_offset() is preferred above tell() by the libbfio layer used in libyal.
  def get_offset(self):
    """Retrieves the current offset into the file input/output (IO) object.

    Returns:
      int: current offset into the file input/output (IO) object.
    """
    return self._fsext_extended_attribute.get_offset()

  # Pythonesque alias for get_offset().
  def tell(self):
    """Retrieves the current offset into the file input/output (IO) object."""
    return self.get_offset()

  def get_size(self):
    """Retrieves the size of the file input/output (IO) object.

    Returns:
      int: size of the file input/output (IO) object.
    """
    return self._fsext_extended_attribute.get_size()

  def seekable(self):
    """Determines if a file input/output (IO) object is seekable.

    Returns:
      bool: True since a file IO object provides a seek method.
    """
    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for a partition file system implementation using pytsk3."""

import unittest

from dfvfs.lib import definitions
from dfvfs.path import factory as path_spec_factory
from dfvfs.resolver import context
from dfvfs.vfs import tsk_partition_file_system

from tests import test_lib as shared_test_lib


class TSKPartitionFileSystemTest(shared_test_lib.BaseTestCase):
  """Tests the TSK partition file system."""

  def setUp(self):
    """Sets up the needed objects used throughout the test."""
    self._resolver_context = context.Context()
    test_path = self._GetTestFilePath(['mbr.raw'])
    self._SkipIfPathNotExists(test_path)

    self._os_path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_OS, location=test_path)
    self._tsk_partition_path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_TSK_PARTITION, location='/',
        parent=self._os_path_spec)

  def tearDown(self):
    """Cleans up the needed objects used throughout the test."""
    self._resolver_context.Empty()

  # mmls test_data/mbr.raw
  # DOS Partition Table
  # Offset Sector: 0
  # Units are in 512-byte sectors
  #
  #      Slot    Start        End          Length       Description
  # 00:  Meta    0000000000   0000000000   0000000001   Primary Table (#0)
  # 01:  -----   0000000000   0000000000   0000000001   Unallocated
  # 02:  00:00   0000000001   0000000350   0000000350   Linux (0x83)
  # 03:  Meta    0000000351   0000002879   0000002529   DOS Extended (0x05)
  # 04:  Meta    0000000351   0000000351   0000000001   Extended Table (#1)
  # 05:  -----   0000000351   0000000351   0000000001   Unallocated
  # 06:  01:00   0000000352   0000002879   0000002528   Linux (0x83)

  def testOpenAndClose(self):
    """Test the open and close functionality."""
    file_system = tsk_partition_file_system.TSKPartitionFileSystem(
        self._resolver_context, self._tsk_partition_path_spec)
    self.assertIsNotNone(file_system)

    file_system.Open()

  def testFileEntryExistsByPathSpec(self):
    """Test the file entry exists by path specification functionality."""
    file_system = tsk_partition_file_system.TSKPartitionFileSystem(
        self._resolver_context, self._tsk_partition_path_spec)
    self.assertIsNotNone(file_system)

    file_system.Open()

    path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_TSK_PARTITION, location='/',
        parent=self._os_path_spec)
    self.assertTrue(file_system.FileEntryExistsByPathSpec(path_spec))

    path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_TSK_PARTITION, part_index=3,
        parent=self._os_path_spec)
    self.assertTrue(file_system.FileEntryExistsByPathSpec(path_spec))

    path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_TSK_PARTITION, part_index=6,
        parent=self._os_path_spec)
    self.assertTrue(file_system.FileEntryExistsByPathSpec(path_spec))

    path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_TSK_PARTITION, location='/p2',
        parent=self._os_path_spec)
    self.assertTrue(file_system.FileEntryExistsByPathSpec(path_spec))

    path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_TSK_PARTITION, part_index=9,
        parent=self._os_path_spec)
    self.assertFalse(file_system.FileEntryExistsByPathSpec(path_spec))

    path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_TSK_PARTITION, location='/p0',
        parent=self._os_path_spec)
    self.assertFalse(file_system.FileEntryExistsByPathSpec(path_spec))

    path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_TSK_PARTITION, location='/p9',
        parent=self._os_path_spec)
    self.assertFalse(file_system.FileEntryExistsByPathSpec(path_spec))

  def testGetFileEntryByPathSpec(self):
    """Tests the GetFileEntryByPathSpec function."""
    file_system = tsk_partition_file_system.TSKPartitionFileSystem(
        self._resolver_context, self._tsk_partition_path_spec)
    self.assertIsNotNone(file_system)

    file_system.Open()

    path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_TSK_PARTITION, location='/',
        parent=self._os_path_spec)
    file_entry = file_system.GetFileEntryByPathSpec(path_spec)

    self.assertIsNotNone(file_entry)
    self.assertEqual(file_entry.name, '')

    path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_TSK_PARTITION, part_index=3,
        parent=self._os_path_spec)
    file_entry = file_system.GetFileEntryByPathSpec(path_spec)

    self.assertIsNotNone(file_entry)
    self.assertEqual(file_entry.name, '')

    path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_TSK_PARTITION, part_index=6,
        parent=self._os_path_spec)
    file_entry = file_system.GetFileEntryByPathSpec(path_spec)

    self.assertIsNotNone(file_entry)
    self.assertEqual(file_entry.name, 'p2')

    path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_TSK_PARTITION, location='/p2',
        parent=self._os_path_spec)
    file_entry = file_system.GetFileEntryByPathSpec(path_spec)

    self.assertIsNotNone(file_entry)
    self.assertEqual(file_entry.name, 'p2')

    path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_TSK_PARTITION, part_index=9,
        parent=self._os_path_spec)
    file_entry = file_system.GetFileEntryByPathSpec(path_spec)

    self.assertIsNone(file_entry)

    path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_TSK_PARTITION, location='/p0',
        parent=self._os_path_spec)
    file_entry = file_system.GetFileEntryByPathSpec(path_spec)

    self.assertIsNone(file_entry)

    path_spec = path_spec_factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_TSK_PARTITION, location='/p9',
        parent=self._os_path_spec)
    file_entry = file_system.GetFileEntryByPathSpec(path_spec)

    self.assertIsNone(file_entry)

  def testGetRootFileEntry(self):
    """Test the get root file entry functionality."""
    file_system = tsk_partition_file_system.TSKPartitionFileSystem(
        self._resolver_context, self._tsk_partition_path_spec)
    self.assertIsNotNone(file_system)

    file_system.Open()

    file_entry = file_system.GetRootFileEntry()

    self.assertIsNotNone(file_entry)
    self.assertEqual(file_entry.name, '')


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The APFS format analyzer helper implementation."""

from dfvfs.analyzer import analyzer
from dfvfs.analyzer import analyzer_helper
from dfvfs.analyzer import specification
from dfvfs.lib import definitions


class APFSAnalyzerHelper(analyzer_helper.AnalyzerHelper):
  """APFS analyzer helper."""

  FORMAT_CATEGORIES = frozenset([
      definitions.FORMAT_CATEGORY_FILE_SYSTEM])

  TYPE_INDICATOR = definitions.TYPE_INDICATOR_APFS

  def GetFormatSpecification(self):
    """Retrieves the format specification.

    Returns:
      FormatSpecification: format specification or None if the format cannot
          be defined by a specification object.
    """
    format_specification = specification.FormatSpecification(
        self.type_indicator)

    # APFS file system signature.
    format_specification.AddNewSignature(b'APSB', offset=32)

    return format_specification


analyzer.Analyzer.RegisterHelper(APFSAnalyzerHelper())



# -*- coding: utf-8 -*-
"""The Volume Shadow Snapshots (VSS) file-like object implementation."""

import os

from dfvfs.file_io import file_io
from dfvfs.lib import errors
from dfvfs.lib import vshadow_helper
from dfvfs.resolver import resolver


class VShadowFile(file_io.FileIO):
  """File input/output (IO) object using pyvshadow."""

  def __init__(self, resolver_context, path_spec):
    """Initializes a file input/output (IO) object.

    Args:
      resolver_context (Context): resolver context.
      path_spec (PathSpec): a path specification.
    """
    super(VShadowFile, self).__init__(resolver_context, path_spec)
    self._file_system = None
    self._vshadow_store = None

  def _Close(self):
    """Closes the file-like object."""
    self._vshadow_store = None

    self._file_system = None

  def _Open(self, mode='rb'):
    """Opens the file-like object defined by path specification.

    Args:
      mode (Optional[str]): file access mode.

    Raises:
      AccessError: if the access to open the file was denied.
      IOError: if the file-like object could not be opened.
      OSError: if the file-like object could not be opened.
      PathSpecError: if the path specification is incorrect.
    """
    store_index = vshadow_helper.VShadowPathSpecGetStoreIndex(self._path_spec)
    if store_index is None:
      raise errors.PathSpecError(
          'Unable to retrieve store index from path specification.')

    self._file_system = resolver.Resolver.OpenFileSystem(
        self._path_spec, resolver_context=self._resolver_context)
    vshadow_volume = self._file_system.GetVShadowVolume()

    if (store_index < 0 or
        store_index >= vshadow_volume.number_of_stores):
      raise errors.PathSpecError((
          f'Unable to retrieve VSS store: {store_index:d} from path '
          f'specification.'))

    vshadow_store = vshadow_volume.get_store(store_index)
    if not vshadow_store.has_in_volume_data():
      raise IOError((
          f'Unable to open VSS store: {store_index:d} without in-volume '
          f'stored data.'))

    self._vshadow_store = vshadow_store

  # Note: that the following functions do not follow the style guide
  # because they are part of the file-like object interface.
  # pylint: disable=invalid-name

  def read(self, size=None):
    """Reads a byte string from the file-like object at the current offset.

    The function will read a byte string of the specified size or
    all of the remaining data if no size was specified.

    Args:
      size (Optional[int]): number of bytes to read, where None is all
          remaining data.

    Returns:
      bytes: data read.

    Raises:
      IOError: if the read failed.
      OSError: if the read failed.
    """
    if not self._is_open:
      raise IOError('Not opened.')

    return self._vshadow_store.read(size)

  def seek(self, offset, whence=os.SEEK_SET):
    """Seeks to an offset within the file-like object.

    Args:
      offset (int): offset to seek to.
      whence (Optional(int)): value that indicates whether offset is an absolute
          or relative position within the file.

    Raises:
      IOError: if the seek failed.
      OSError: if the seek failed.
    """
    if not self._is_open:
      raise IOError('Not opened.')

    self._vshadow_store.seek(offset, whence)

  def get_offset(self):
    """Retrieves the current offset into the file-like object.

    Returns:
      int: current offset into the file-like object.

    Raises:
      IOError: if the file-like object has not been opened.
      OSError: if the file-like object has not been opened.
    """
    if not self._is_open:
      raise IOError('Not opened.')

    return self._vshadow_store.get_offset()

  def get_size(self):
    """Retrieves the size of the file-like object.

    Returns:
      int: size of the file-like object data.

    Raises:
      IOError: if the file-like object has not been opened.
      OSError: if the file-like object has not been opened.
    """
    if not self._is_open:
      raise IOError('Not opened.')

    return self._vshadow_store.volume_size



# -*- coding: utf-8 -*-
"""The Volume Shadow Snapshots (VSS) directory implementation."""

from dfvfs.path import vshadow_path_spec
from dfvfs.vfs import directory


class VShadowDirectory(directory.Directory):
  """File system directory that uses pyvshadow."""

  def _EntriesGenerator(self):
    """Retrieves directory entries.

    Since a directory can contain a vast number of entries using
    a generator is more memory efficient.

    Yields:
      VShadowPathSpec: a path specification.
    """
    location = getattr(self.path_spec, 'location', None)
    store_index = getattr(self.path_spec, 'store_index', None)

    # Only the virtual root file has directory entries.
    if (store_index is None and location is not None and
        location == self._file_system.LOCATION_ROOT):
      vshadow_volume = self._file_system.GetVShadowVolume()

      for store_index in range(0, vshadow_volume.number_of_stores):
        vss_store_index = store_index + 1
        yield vshadow_path_spec.VShadowPathSpec(
            location=f'/vss{vss_store_index:d}', parent=self.path_spec.parent,
            store_index=store_index)



# -*- coding: utf-8 -*-
"""Imports for path specification factory."""

from dfvfs.path import apfs_container_path_spec
from dfvfs.path import apfs_path_spec
from dfvfs.path import bde_path_spec
from dfvfs.path import compressed_stream_path_spec
from dfvfs.path import cpio_path_spec
from dfvfs.path import cs_path_spec
from dfvfs.path import data_range_path_spec
from dfvfs.path import encoded_stream_path_spec
from dfvfs.path import encrypted_stream_path_spec
from dfvfs.path import ewf_path_spec
from dfvfs.path import ext_path_spec
from dfvfs.path import fake_path_spec
from dfvfs.path import fat_path_spec
from dfvfs.path import gpt_path_spec
from dfvfs.path import gzip_path_spec
from dfvfs.path import hfs_path_spec
from dfvfs.path import lvm_path_spec
from dfvfs.path import modi_path_spec
from dfvfs.path import mount_path_spec
from dfvfs.path import ntfs_path_spec
from dfvfs.path import os_path_spec
from dfvfs.path import path_spec
from dfvfs.path import phdi_path_spec
from dfvfs.path import qcow_path_spec
from dfvfs.path import raw_path_spec
from dfvfs.path import sqlite_blob_path_spec
from dfvfs.path import tar_path_spec
from dfvfs.path import tsk_path_spec
from dfvfs.path import tsk_partition_path_spec
from dfvfs.path import vhdi_path_spec
from dfvfs.path import vmdk_path_spec
from dfvfs.path import vshadow_path_spec
from dfvfs.path import xfs_path_spec
from dfvfs.path import zip_path_spec



# -*- coding: utf-8 -*-
"""The GUID Partition Table (GPT) volume system."""

from dfvfs.lib import definitions
from dfvfs.volume import factory
from dfvfs.volume import volume_system


class GPTVolume(volume_system.Volume):
  """Volume that uses pyvsgpt."""

  def __init__(self, file_entry):
    """Initializes a GPT volume.

    Args:
      file_entry (GPTFileEntry): a GPT file entry.
    """
    super(GPTVolume, self).__init__(file_entry.name)
    self._file_entry = file_entry

  def _Parse(self):
    """Extracts attributes and extents from the volume."""
    vsgpt_partition = self._file_entry.GetGPTPartition()

    volume_attribute = volume_system.VolumeAttribute(
        'identifier', vsgpt_partition.identifier)
    self._AddAttribute(volume_attribute)

    volume_extent = volume_system.VolumeExtent(
        vsgpt_partition.volume_offset, vsgpt_partition.size)
    self._extents.append(volume_extent)


class GPTVolumeSystem(volume_system.VolumeSystem):
  """Volume system that uses pyvsgpt."""

  TYPE_INDICATOR = definitions.TYPE_INDICATOR_GPT

  VOLUME_IDENTIFIER_PREFIX = 'p'

  def _Parse(self):
    """Extracts sections and volumes from the volume system."""
    root_file_entry = self._file_system.GetRootFileEntry()

    for sub_file_entry in root_file_entry.sub_file_entries:
      volume = GPTVolume(sub_file_entry)
      self._AddVolume(volume)


factory.Factory.RegisterVolumeSystem(GPTVolumeSystem)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the storage media RAW image support helper functions."""

import unittest

from dfvfs.lib import raw_helper
from dfvfs.path import fake_path_spec
from dfvfs.path import raw_path_spec
from dfvfs.resolver import context
from dfvfs.vfs import fake_file_system

from tests import test_lib as shared_test_lib


class GlobRawFileTest(shared_test_lib.BaseTestCase):
  """The unit test for the storage media RAW image file glob functionality."""

  def _BuildFileFakeFileSystem(
      self, segment_filenames, segment_file_path_specs):
    """Builds a fake file system containing storage media RAW segment files.

    Args:
      segment_filenames (list[str]): segment filenames.
      segment_file_path_specs (list[PathSpec]): resulting segment file path
          specifications.

    Returns:
      FakeFileSystem: fake file system.
    """
    resolver_context = context.Context()
    path_spec = fake_path_spec.FakePathSpec(location='/')
    file_system = fake_file_system.FakeFileSystem(resolver_context, path_spec)

    for segment_filename in segment_filenames:
      path = f'/{segment_filename:s}'

      file_system.AddFileEntry(path)
      segment_file_path_specs.append(fake_path_spec.FakePathSpec(location=path))

    return file_system

  def testGlobRawSinglecExtension(self):
    """Test the glob function for a RAW single extension scheme."""
    # Test single segment file: dd.
    segment_filenames = ['ímynd.dd']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    path_spec = fake_path_spec.FakePathSpec(location='/ímynd.dd')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test single segment file: dmg.
    segment_filenames = ['image.dmg']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    path_spec = fake_path_spec.FakePathSpec(location='/image.dmg')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test single segment file: img.
    segment_filenames = ['image.img']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    path_spec = fake_path_spec.FakePathSpec(location='/image.img')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test single segment file: raw.
    segment_filenames = ['image.raw']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    path_spec = fake_path_spec.FakePathSpec(location='/image.raw')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

  def testGlobRawAlphabeticalExtension(self):
    """Test the glob function for a RAW alphabetical extension scheme."""
    segment_filenames = ['image.aaa']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    # Test single segment file: aaa.
    path_spec = fake_path_spec.FakePathSpec(location='/image.aaa')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test non exiting segment file: aaa.
    expected_segment_file_path_specs = []

    path_spec = fake_path_spec.FakePathSpec(location='/bogus.aaa')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test multiple segment files: aaa-aak.
    segment_filenames = [
        'image.aaa', 'image.aab', 'image.aac', 'image.aad', 'image.aae',
        'image.aaf', 'image.aag', 'image.aah', 'image.aai', 'image.aaj',
        'image.aak']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    path_spec = fake_path_spec.FakePathSpec(location='/image.aaa')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test multiple segment files: AAA-AAk.
    segment_filenames = [
        'image.AAA', 'image.AAB', 'image.AAC', 'image.AAD', 'image.AAE',
        'image.AAF', 'image.AAG', 'image.AAH', 'image.AAI', 'image.AAJ',
        'image.AAK']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    path_spec = fake_path_spec.FakePathSpec(location='/image.AAA')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

  def testGlobRawAlphabeticalSuffix(self):
    """Test the glob function for a RAW alphabetical suffix scheme."""
    segment_filenames = ['imageaaa']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    # Test single segment file: aaa.
    path_spec = fake_path_spec.FakePathSpec(location='/imageaaa')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test non exiting segment file: aaa.
    expected_segment_file_path_specs = []

    path_spec = fake_path_spec.FakePathSpec(location='/bogusaaa')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test multiple segment files: aaa-aak.
    segment_filenames = [
        'imageaaa', 'imageaab', 'imageaac', 'imageaad', 'imageaae',
        'imageaaf', 'imageaag', 'imageaah', 'imageaai', 'imageaaj',
        'imageaak']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    path_spec = fake_path_spec.FakePathSpec(location='/imageaaa')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test multiple segment files: AAA-AAk.
    segment_filenames = [
        'imageAAA', 'imageAAB', 'imageAAC', 'imageAAD', 'imageAAE',
        'imageAAF', 'imageAAG', 'imageAAH', 'imageAAI', 'imageAAJ',
        'imageAAK']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    path_spec = fake_path_spec.FakePathSpec(location='/imageAAA')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

  def testGlobRawNumericExtension(self):
    """Test the glob function for a RAW numeric extension scheme."""
    segment_filenames = ['image.000']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    # Test single segment file: 000.
    path_spec = fake_path_spec.FakePathSpec(location='/image.000')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test non exiting segment file: 000.
    expected_segment_file_path_specs = []

    path_spec = fake_path_spec.FakePathSpec(location='/bogus.000')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test multiple segment files: 000-010.
    segment_filenames = [
        'image.000', 'image.001', 'image.002', 'image.003', 'image.004',
        'image.005', 'image.006', 'image.007', 'image.008', 'image.009',
        'image.010']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    path_spec = fake_path_spec.FakePathSpec(location='/image.000')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test multiple segment files: 001-010.
    segment_filenames = [
        'image.001', 'image.002', 'image.003', 'image.004', 'image.005',
        'image.006', 'image.007', 'image.008', 'image.009', 'image.010']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    path_spec = fake_path_spec.FakePathSpec(location='/image.001')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test multiple segment files: 1-10.
    segment_filenames = [
        'image.1', 'image.2', 'image.3', 'image.4', 'image.5',
        'image.6', 'image.7', 'image.8', 'image.9', 'image.10']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    path_spec = fake_path_spec.FakePathSpec(location='/image.1')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

  def testGlobRawNumericSuffix(self):
    """Test the glob function for a RAW numeric suffix scheme."""
    segment_filenames = ['image1']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    # Test single segment file: 000.
    path_spec = fake_path_spec.FakePathSpec(location='/image1')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test non exiting segment file: 000.
    expected_segment_file_path_specs = []

    path_spec = fake_path_spec.FakePathSpec(location='/bogus1')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test multiple segment files: 000-010.
    segment_filenames = [
        'image0', 'image1', 'image2', 'image3', 'image4', 'image5',
        'image6', 'image7', 'image8', 'image9', 'image10']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    path_spec = fake_path_spec.FakePathSpec(location='/image0')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test multiple segment files: 1-10.
    segment_filenames = [
        'image1', 'image2', 'image3', 'image4', 'image5',
        'image6', 'image7', 'image8', 'image9', 'image10']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    path_spec = fake_path_spec.FakePathSpec(location='/image1')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test multiple segment files: 001-010.
    segment_filenames = [
        'image001', 'image002', 'image003', 'image004', 'image005',
        'image006', 'image007', 'image008', 'image009', 'image010']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    path_spec = fake_path_spec.FakePathSpec(location='/image001')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

  def testGlobRawAsbExtension(self):
    """Test the glob function for a RAW ASB extension scheme."""
    segment_filenames = ['image001.asb']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    # Test single segment file: 001.
    path_spec = fake_path_spec.FakePathSpec(location='/image001.asb')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test non exiting segment file: 001.
    expected_segment_file_path_specs = []

    path_spec = fake_path_spec.FakePathSpec(location='/bogus000.asb')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test multiple segment files: 001-010.
    segment_filenames = [
        'image001.asb', 'image002.asb', 'image003.asb', 'image004.asb',
        'image005.asb', 'image006.asb', 'image007.asb', 'image008.asb',
        'image009.asb', 'image010.asb']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    path_spec = fake_path_spec.FakePathSpec(location='/image001.asb')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

  def testGlobRawVMDKExtension(self):
    """Test the glob function for a RAW VMDK extension scheme."""
    segment_filenames = ['image-f001.vmdk']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    # Test single segment file: 001.
    path_spec = fake_path_spec.FakePathSpec(location='/image-f001.vmdk')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test non exiting segment file: 001.
    expected_segment_file_path_specs = []

    path_spec = fake_path_spec.FakePathSpec(location='/bogus-f000.vmdk')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)

    # Test multiple segment files: 001-010.
    segment_filenames = [
        'image-f001.vmdk', 'image-f002.vmdk', 'image-f003.vmdk',
        'image-f004.vmdk', 'image-f005.vmdk', 'image-f006.vmdk',
        'image-f007.vmdk', 'image-f008.vmdk', 'image-f009.vmdk',
        'image-f010.vmdk']
    expected_segment_file_path_specs = []
    file_system = self._BuildFileFakeFileSystem(
        segment_filenames, expected_segment_file_path_specs)

    path_spec = fake_path_spec.FakePathSpec(location='/image-f001.vmdk')
    path_spec = raw_path_spec.RawPathSpec(parent=path_spec)

    segment_file_path_specs = raw_helper.RawGlobPathSpec(file_system, path_spec)
    self.assertEqual(
        len(segment_file_path_specs), len(expected_segment_file_path_specs))
    self.assertEqual(
        segment_file_path_specs, expected_segment_file_path_specs)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the key chain."""

import unittest

from dfvfs.credentials import keychain
from dfvfs.lib import definitions
from dfvfs.path import factory

from tests import test_lib as shared_test_lib


class KeychainTest(shared_test_lib.BaseTestCase):
  """Tests the key chain."""

  # TODO: add tests for Empty
  # TODO: add tests for ExtractCredentialsFromPathSpec

  def testCredentialGetSet(self):
    """Tests the GetCredential and SetCredential functions."""
    test_keychain = keychain.KeyChain()

    fake_path_spec = factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_FAKE, location='/test')
    bde_path_spec = factory.Factory.NewPathSpec(
        definitions.TYPE_INDICATOR_BDE, parent=fake_path_spec)

    with self.assertRaises(AttributeError):
      test_keychain.SetCredential(fake_path_spec, 'password', 'TEST')

    test_keychain.SetCredential(bde_path_spec, 'password', 'TEST')

    credential = test_keychain.GetCredential(fake_path_spec, 'password')
    self.assertIsNone(credential)

    credential = test_keychain.GetCredential(bde_path_spec, 'password')
    self.assertEqual(credential, 'TEST')

    credentials = test_keychain.GetCredentials(bde_path_spec)
    self.assertEqual(credentials, {'password': 'TEST'})


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the Core Storage (CS) credentials."""

import unittest

from dfvfs.credentials import cs_credentials

from tests import test_lib as shared_test_lib


class CSCredentials(shared_test_lib.BaseTestCase):
  """Tests the Core Storage (CS) credentials."""

  def testInitialize(self):
    """Tests the __init__ function."""
    test_credentials = cs_credentials.CSCredentials()
    self.assertIsNotNone(test_credentials)


if __name__ == '__main__':
  unittest.main()
